#!/usr/bin/python3
# coding: utf8 
import random
import os
import modisSuite as ms
import shutil
class imageModis:
    def __init__(self,files,*arg,**args):
        self.files=files
    def returnHDF(self):
        ls=[]
        for f in self.files:
            if f.typ=="hdf":
                ls.append(f)
        return ls
    def mosaic(self,mosName=""):
        if mosName=="":
            mosName=os.path.join(os.path.dirname(self.files[0].path),'%f.hdf'%random.random())
        arg=[f.path for f in self.returnHDF()]+[mosName,]
        ms.mosaic(*arg)
        return imageModis([Afile("","","","hdf",mosName),])
    def clip(self,pixel=(0,0)):
        return imageModis([f.clip(pixel=pixel) for f in self.files])
    def subset(self,sublist=[]):
        return imageModis([f.subset(sublist=sublist) for f in self.files])
    def compress(self,*args,**kargs):
        return imageModis([f.compress(*args) for f in self.files])
    def delete(self):
        for f in self.files:
            f.delete()
    def move(self,newfolder):
        for f in self.files:
            f.move(newfolder)
class Afile:
    def __init__(self,name,date,link,typ,path,*arg,**args):
        self.name=name
        self.date=date
        self.typ=typ
        self.link=link
        self.path=path
        self.size=0
        self.tile=0
        self.product=0
        self.dateTelechargement=""
        self.telecharge=True
    def delete(self):
        os.remove(self.path)
    def move(self,newPath):
        # Should do something if newPath is a folder
        os.rename(self.path,newPath)
        self.path=newPath
    # Should add copy function
    def clip(self,clipName="",**args):
        if clipName=="":
            clipName=os.path.join(os.path.dirname(self.path),'%f.hdf'%random.random())
        ms.clip(self.path,clipName,pixel=args["pixel"])
        return Afile("","","","hdf",clipName)
    def subset(self,subName="",sublist=[],**args):
        if subName=="":
            subName=os.path.join(os.path.dirname(self.path),'%f.hdf'%random.random())
        ms.subset(self.path,subName,*sublist)
        return Afile("","","","hdf",subName)
    def compress(self,*args,**kargs):
        ms.compress(self.path,args)



#!/usr/bin/python3
# coding: utf8 

# Importations
import numpy as np


# Function to convert points
def cartToSin(xy,diameter):
	def conv(x,y,diameter):
		# Convert radient coordinates to sin
		phy=y/diameter
		lamb=x/diameter
		return (lamb*np.cos(phy),phy)
	if type(xy) == tuple:
		try:
			x,y = xy
		except ValueError:
			# Too much or not enough value in tupple
			return None
		return conv(x,y,diameter)
		
	elif type(xy) == list:
		lcon=[]
		for co in xy:
			try:
				x,y = co
			except ValueError:
				# Too much or not enough value in tupple
				return None
			lcon.append(conv(x,y,diameter))
		return lcon
def sinToCart(xy,diameter):
	def conv(x,y,diameter):
		# Convert radient sin to coordinates
		x/=(diameter)
		y/=(diameter)
		return (x/(np.cos(y)),y)
	if type(xy) == tuple:
		try:
			x,y = xy
		except ValueError:
			# Too much or not enough value in tupple
			return None
		return conv(x,y,diameter)
		
	elif type(xy) == list:
		lcon=[]
		for co in xy:
			try:
				x,y = co
			except ValueError:
				# Too much or not enough value in tupple
				return None
			lcon.append(conv(x,y,diameter))
		return lcon
   
def cooToPixel(xy,boundcoordinate,numPixel):
	return



# coding: utf8
# Importations
from pyhdf.SD import *
import pyhdf.error 
from pyhdf.V import *
from pyhdf.HDF import *
from pyhdf.VS import *
import re
import numpy as np
from modisSuite import logMod


Nom="modisSuite Clip"
# Defining the class to manage attributes from the HDF file
class atribute:
	def __init__(self,raw,name="",dsi=[0,]):
		#print(dsi)
		self.name=name
		self.raw=raw
		self.GROUP = {}
		self.OBJECT = {}
		self.OBJECTindex = {}
		self.variable = {}
		colect=""
		colecting=False
		end=""
		lines=self.raw.split('\n')
		progb=re.compile('GROUP=[A-Za-z_0-9]{1,}')
		proge=re.compile('END_GROUP=[A-Za-z_0-9]{1,}')
		progob=re.compile('OBJECT=[A-Za-z_0-9]{1,}')
		progoe=re.compile('END_OBJECT=[A-Za-z_0-9]{1,}')
		for line in lines:
			#Add group
			b=progb.findall(line)
			e=proge.findall(line)
			ob=progob.findall(line)
			oe=progoe.findall(line)
			if (len(e)==0 and len(b)>0) and (not colecting):
				# Begin collecting
				colecting=True
				colect=""
				end=b[0]
				#print(type(e),type(b),type(line))
			elif (len(oe)==0 and len(ob)>0) and (not colecting):
				# Begin collecting
				colecting=True
				colect=""
				end=ob[0]
				#print(type(e),type(b),type(line))
			elif len(e)>0 and len(b)>0 and line.find(end)>-1:
				colecting=False
				# End collecting
				name=end.split("=")[-1]
				self.GROUP[name]=atribute(colect,dsi=dsi)
			elif len(oe)>0 and len(ob)>0 and line.find(end)>-1:
				colecting=False
				# End collecting
				name=end.split("=")[-1]
				self.OBJECTindex[name] = dsi[0]
				dsi[0]+=1
				self.OBJECT[name] = atribute(colect,dsi=dsi)
			elif colecting:
				# Collecting
				# print(line)
				colect+=line+"\n"
			elif  len(b)==0 and len(ob)==0:
				# adding to variable
				try:
					#print(line)
					self.variable[line.split("=")[0].strip()]=line.split("=")[1]
				except IndexError:
					pass
			
	def listgrid(self):
	# list the grids in the group
		try:
			gs=self.GROUP["GridStructure"]
			gg=list(gs.GROUP.keys())
			return len(gg),gg
		except KeyError:
			return -1,[]
	def listgridname(self):
		# list the grids name in the group
		try:
			gs=self.GROUP["GridStructure"]
			gg=[gs.GROUP[x].variable["GridName"] for x in gs.GROUP]
			return len(gg),gg
		except KeyError:
			return -1,[]
	def listdataset(self):
		#Aller chercher les grids
		gr=self.listgrid()[1]
		ds=[]
		for g in gr:
			ds+=list(self.GROUP["GridStructure"].GROUP[g].GROUP["DataField"].OBJECT.keys())
		return ds
	def listdatasetbyname(self):
		#get ds by name
		gr=self.listgrid()[1]
		ds=[]
		for g in gr:
			dsg=self.GROUP["GridStructure"].GROUP[g].GROUP["DataField"].OBJECT
			ds+=[dsg[x].variable["DataFieldName"] for x in dsg]
		return ds
	def getgridbyname(self,name):
		gs=self.GROUP["GridStructure"]
		for x in gs.GROUP:
			if gs.GROUP[x].variable["GridName"] == name:
				return gs.GROUP[x]
		return None
	def orderedDS(self):
		#get ds by name
		gr=self.listgrid()[1]
		ds=self.listdatasetbyname()
		#print(gr)
		for g in gr:
			dsg = self.GROUP["GridStructure"].GROUP[g].GROUP["DataField"]
			i = dsg.OBJECTindex
			ii = dsg.OBJECT
			#print("haha",ds)
			for e in i:
				#print(i[e])
				ds[i[e]] = ii[e].variable["DataFieldName"]
		return ds
	def __str__(self):
		st=""
		# Ajouter les variables
		for x in self.variable:
			st+=x+"="+self.variable[x]+"\n"
		# Ajouter les group
		
		for x in self.GROUP:
			st+="GROUP="+x+"\n"
			
			gg=str(self.GROUP[x]).split("\n")
			gg.remove("")
			st+="\n".join(["\t"+h for h in gg])+"\n"
			st+="END_GROUP="+x+"\n"
		# Ajouter les object
		for x in self.OBJECT:
			st+="OBJECT="+x+"\n"
			gg=str(self.OBJECT[x]).split("\n")
			gg.remove("")
			st+="\n".join(["\t"+h for h in gg])+"\n"
			st+="END_OBJECT="+x+"\n"
		return st
        





def clip(*arg,**args):
	# clip(file in, file out,pixel=[upper pixel, leftmost pixel, lower pixel,rightmost pixel])
	# This function clip the file and produce new file
	oldfile = arg[0]
	newfilename = arg[1]
	coo = args["pixel"]
	
	try:
		log=args["log"]
	except KeyError:
		log=logMod.Log("",nolog=True)
	
	
	UPC = coo[0]
	LMC = coo[1]
	LOC = coo[2]
	RMC = coo[3]
    

	# Should eventually check if files exists and can be read ***IMPROVE***
	try:
		HDFF = SD(oldfile,SDC.READ)						# This is the list of the FILES to merge
	except TypeError:
		HDFF = SD(oldfile.encode('ascii','ignore'),SDC.READ)						# This is the list of the FILES to merge
	
	
	# This is the list of the ATTRIBUTE "StructMetadata.0" of the files
	attOfF = atribute(HDFF.attributes()["StructMetadata.0"],oldfile,dsi=[0,])

		
	
	
	## Listing all the GRIDS that the new file will have

	gridlist = attOfF.listgridname()[1]		# listgridname return a list of all the grids name




	## Listing all the DATASETS that the new file will have
	
	# Should check if any grid ***IMPROVE***
	dslist = attOfF.orderedDS()


	
	## Validation of commoun information
###############################################################################
# Some informations have to be the same for each file or else no mosaic can   #
# be made for exemple two files with not the same projection type can't be    #
# merged together. ***IMPROVE*** Maybe in the future we could transform file  #
# so that they have the same projection or some other thing.                  #
###############################################################################

	# List of parameter to check to insure that they are the same
	paramMustSim = ["Projection","ProjParams","SphereCode"]
	# Dictionary that will keep all the informations about every file
	paramMustSimDict = {}

	for grid in gridlist:
		# Verification of a grid
		
		paramdict = {}		# Dictionary that keep the actual value that have to be the same
		
		
		bigG = attOfF.getgridbyname(grid)		# Getting all the attributes in the grid of a file
		if bigG is not None:
			# If the grid exists in that file
			
			
			for p in paramMustSim:
				# Checking every parameters that must be the same
				paramdict[p] = bigG.variable[p]
				
			# Validation of same Dtype for each datafield
			go = bigG.GROUP["DataField"].OBJECT
			for r in go:
				paramdict[go[r].variable["DataFieldName"]]=go[r].variable["DataType"]
			
						
		# Keep all this info for later it's going to be useful
		paramMustSimDict[grid]=paramdict
			
			

# LAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA

	## Determination of new informations
###############################################################################
# Some properties have to be calculated in order to merge. This section is    #
# doing just that                                                             #
###############################################################################


	gridResolX={}			# Getting the RESOLUTION in the X direction for each grid
	gridResolY={}			# Getting the RESOLUTION in the Y direction for each grid
	extremeup={}			# Getting the UPPER coordinates for each grid
	extremedown={}			# Getting the LOWEST coordinates for each grid
	extremeleft={}			# Getting the LEFTMOST coordinates for each grid
	extremeright={}			# Getting the RIGHTMOST coordinates for each grid
	gridDimX={}				# Getting the DIMENSIONS of X direction for each grid
	gridDimY={}				# Getting the DIMENSIONS of Y direction for each grid
	NoValueDS={}			# Getting the fill value of each dataset
	dtypeDS={}				# Getting the DTYPE for each dataset
	dstogrid={}				# Knowing wich is the GRID for each dataset
	filGridULC={}			# Getting the upper left corner of each file for each grid

	for grid in gridlist:
		# For each grid
		filGridULC[grid]={}			# Adding a dictionary for each grid that will contain information on every file
		
		### Determination of resolution of each grid
		# ***IMPROVE*** Should check if bigd is none
		bigG=attOfF.getgridbyname(grid)				# Getting all the attributes in the grid of a file
		
		# Get extreme grid point
		ulp=eval(bigG.variable["UpperLeftPointMtrs"])
		lrp=eval(bigG.variable["LowerRightMtrs"])
		
		# Get grid dimmension (PIXEL)
		dimx=int(bigG.variable["XDim"])
		dimy=int(bigG.variable["YDim"])
		
		# Calculate grid resolution
		gridResolX[grid]=(lrp[0]-ulp[0])/dimx
		gridResolY[grid]=(ulp[1]-lrp[1])/dimy
		
		### Determination of new extreme coordinates for each grid
		# up
		extremeup[grid]    = ulp[1] - gridResolY[grid] * UPC
		# down
		extremedown[grid]  = lrp[1] + gridResolY[grid] * (dimy - LOC)
		# left
		extremeleft[grid]  = ulp[0] + gridResolX[grid] * LMC
		# right
		extremeright[grid] = lrp[0] - gridResolX[grid] * (dimx - RMC)
		#print("\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n",int(LOC - UPC),int(RMC - LMC),"\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n\n")
		
		
		
		### Detetermination of dataset to grid name
		if bigG is not None:
			go=bigG.GROUP["DataField"].OBJECT
			for r in go:
				dstogrid[ go[r].variable["DataFieldName"] ] = grid
				
		### Determination of new extreme coordinates for each grid
		
		## Determination of ULC for each grid in each file
		filGridULC[grid][attOfF.name] = ulp
		## determination of new dimension for each grid
		gridDimY[grid] = int(LOC - UPC)
		gridDimX[grid] = int(RMC - LMC)

	for ds in dslist:
		# For each dataset
		
		# For each hdf file
		
		# Try opening dataset
		try:
			sds = HDFF.select(eval(ds))
			# Get fill value
			NoValueDS[ds] = sds.getfillvalue()
			# Get dtype
			dtypeDS[ds] = sds.info()[3]
		except:
			log.log('e',Nom,"no dataset")
	


	## Start creating new file
###############################################################################
# This is the actual part were stuf appens                                    #
###############################################################################

	# This part is the same for every file in any circumstances
	########## absolute ########################
	
	# Open new file
	try:
		hdf = HDF(newfilename, HC.WRITE  | HC.CREATE  |HC.TRUNC)
		sd  =  SD(newfilename, SDC.WRITE | SDC.CREATE )
	except TypeError:
		hdf = HDF(newfilename.encode('ascii','ignore'), HC.WRITE  | HC.CREATE  |HC.TRUNC)
		sd  =  SD(newfilename.encode('ascii','ignore'), SDC.WRITE | SDC.CREATE )
	v=hdf.vgstart()
	vg={}
	vg1={}
	vg2={}
		
	## rewrite the gridlist
	gridlist = []
	for ds in dslist:
		if dstogrid[ds] not in gridlist:
			gridlist.append(dstogrid[ds])
	
	for grid in gridlist:
		vg[grid]=v.attach(-1,write=1)
		vg[grid]._class="GRID"
		vg[grid]._name=eval(grid)
		vg1[grid]=v.attach(-1,write=1)
		vg2[grid]=v.attach(-1,write=1)
		vg1[grid]._class="GRID Vgroup"
		vg1[grid]._name="Data Fields"
		vg2[grid]._class="GRID Vgroup"
		vg2[grid]._name="Grid Attributes"
		vg[grid].insert(vg1[grid])
		vg[grid].insert(vg2[grid])
	########## absolute ########################


	# Create dataset with the right size
	for ds in dslist:
		theGrid=dstogrid[ds]
		# Get grid name of data set
		sds = sd.create(eval(ds),dtypeDS[ds],(gridDimY[theGrid],gridDimX[theGrid]))
		
		# Set fill value
		fv=NoValueDS[ds]
		try:
			sds.setfillvalue(NoValueDS[ds])
		except OverflowError:
			log.log('e',Nom,"dataset fill value")
			sds.setfillvalue(0)
		## write real data

		try:
			# Determine were the data will be writen
			ulc = filGridULC[theGrid][attOfF.name]
			# Determine the position on the grid
			y = (extremeup[theGrid]-ulc[1])/(extremeup[theGrid]-extremedown[theGrid])
			x = (ulc[0]-extremeleft[theGrid])/(extremeright[theGrid]-extremeleft[theGrid])
			y = int(y*gridDimY[theGrid])
			x = int(x*gridDimX[theGrid])
			# read data from files
			osds = HDFF.select(eval(ds))
			sh = osds[:].shape
			sds[0:int(LOC-UPC),0:int(RMC-LMC)] = osds[UPC:LOC,LMC:RMC]
			
			
			osds.endaccess()
		except:
			pass
		# Close sds
		vg1[dstogrid[ds]].add(HC.DFTAG_NDG,sds.ref())
		sds.endaccess()



	for g in vg1:
		vg1[g].detach()
		vg2[g].detach()
		vg[g].detach()

	# Create attribute table for the file
	attstr="GROUP=GridStructure\n"
	gridcount=1
	for gr in gridlist:
		# Start group grid
		attstr+="\tGROUP=GRID_%i\n"%gridcount
		# Add grid name
		attstr+="\t\tGridName=%s\n"%gr
		# Add dimention
		attstr+="\t\tXDim=%i\n"%gridDimX[gr]
		attstr+="\t\tYDim=%i\n"%gridDimY[gr]
		# Add UpperLeftPointMtrs
		attstr+="\t\tUpperLeftPointMtrs=(%f,%f)\n"%(extremeleft[gr],extremeup[gr])
		# Add lrp
		attstr+="\t\tLowerRightMtrs=(%f,%f)\n"%(extremeright[gr],extremedown[gr])
		# Add projection
		attstr+="\t\tProjection=%s\n"%paramMustSimDict[gr]["Projection"]
		# ProjParams
		attstr+="\t\tProjParams=%s\n"%paramMustSimDict[gr]["ProjParams"]
		# SphereCode
		attstr+="\t\tSphereCode=%s\n"%paramMustSimDict[gr]["SphereCode"]
		
		attstr+="""\t\tGROUP=Dimension
	\t\tEND_GROUP=Dimension
	\t\tGROUP=DataField\n"""

		## Add data sets
		# create list of ds for current grid
		lsdsgr=[]
		dsnum=1
		for ds in dslist:
			if dstogrid[ds] == gr:
				# Add object
				attstr+="\t\t\tOBJECT=DataField_%i\n"%dsnum
				# datafield name
				attstr+="\t\t\t\tDataFieldName=%s\n"%ds
				# datatype
				attstr+="\t\t\t\tDataType=%s\n"%paramMustSimDict[gr][ds]
				# dim
				attstr+='\t\t\t\tDimList=("YDim","XDim")\n'
				attstr+="\t\t\tEND_OBJECT=DataField_%i\n"%dsnum
				dsnum+=1
		attstr+="\t\tEND_GROUP=DataField\n"
		attstr+="""\t\tGROUP=MergedFields
	\t\tEND_GROUP=MergedFields\n"""
		attstr+="\tEND_GROUP=GRID_%i\n"%gridcount
		gridcount+=1
	attstr+="""END_GROUP=GridStructure
	GROUP=PointStructure
	END_GROUP=PointStructure
	END"""
	# adding attribute to new file
	att=sd.attr('StructMetadata.0')
	att.set(SDC.CHAR,attstr)
	sd.end()
	hdf.close()
	
	# This should return something somehow



#!/usr/bin/python3
# coding: utf8
"""
Auteur: Gabriel Desharnais

Module permetant de télécharger des images depuis les serveurs du NSIDC

Module that download images from NSIDC servers
"""
from . import logMod
from . import imageObject
import requests
import datetime
import os
import sys  
import re

#URL de base du serveur de données
URL="https://n5eil01u.ecs.nsidc.org"
URL2="https://e4ftl01.cr.usgs.gov"
#Préfixe à l'addresse de téléchargement
PreTel='<A HREF="'
#
Nom="Downloader"
class telecharger:
    def __init__(self,produit,utilisateur,motdepasse,date="",delta=0,tuiles=[],log=logMod.Log("",nolog=True),output="",ignore=[],listfile=False):
        self.log=log
        self.tempsRestant=0             #Le temps restant à la tâche
        self.debit=0                    #La vitesse de téléchargement
        self.progression=0              #La progression en pourcentage
        self.produit=produit.upper()
        self.utilisateur=utilisateur
        self.motdepasse=motdepasse
        self.date=date
        self.delta=delta+1
        self.tuiles=tuiles
        self.ignore=[da.replace("-",".") for da in ignore]
        self.session=requests.session()
        self.output=output
        self.authentification(self.session)
        self.listfile=listfile
        log.log('i',Nom,'Objet telecharger créer')
    def listeSource(self):
        liste={}
        def listeSourceUSGS():
            h='<img src="/icons/folder.gif" alt="[DIR]"> <a href="'
            prog=re.compile('\<img src="/icons/folder\.gif" alt="\[DIR\]"\> \<a href=".{1,}/">')
            zztop=self.session.get(URL2).text
            m=prog.findall(zztop)
            for e in m:
                nom=e[len(h):-3]
                #print(nom)
                liste[nom]=URL2+"/"+nom
            
        def listeSourceNSICD():
            
            r=self.session.get(URL)
            rs=r.text
            i=rs.find("NSIDC DATA ARE AVAILABLE VIA HTTPS VIA THE LINKS BELOW.")
            while not rs[i:].find(PreTel)==-1:
                i=rs[i:].find(PreTel)+len(PreTel)+i
                e=rs[i:].find('"')+i
                f=rs[e:].find('/')+e
                g=rs[f:].find(' ')+f
                #raw_input( rs[i:e])
                #print(rs[f+1:g])
                liste[rs[f+1:g]]=rs[i:e-1]
                i=g
        listeSourceUSGS()
        listeSourceNSICD()
        #print(liste)
        return liste
    def listeProduit(self):
        ListeSource=self.listeSource()
        ListeProduit={}
        #print(ListeSource)
        for l in ListeSource:
                try:
                    
                    #get the web page that list all product
                    r=self.session.get(ListeSource[l])
                    rs=r.text
                    #regex that select only product links
                    prog=re.compile('href="[A-Z,a-z,0-9,\.,_]{1,}/"')
                    #find all match of this re
                    m=prog.findall(rs)
                    #Add match to dictionnary
                    for prod in m:
                        ListeProduit[prod[6:-2].upper()]=ListeSource[l]+"/"+prod[6:-2]
                except:
                    #print("oups",l)
                    #print(rs)
                    pass
        return ListeProduit
    def listeToutesDates(self):
        ListeDates={}
        l=self.listeProduit()
        #Get the date list for the product
        r=self.session.get(l[self.produit])
        rs=r.text
        #regex that select only date of format YYYY.MM.DD
        prog=re.compile('[0-9]{4}\.[0-9]{2}\.[0-9]{2}')
        #find all match of this re
        m=prog.findall(rs)
        #Add match to dictionnary
        for date in m:
            ListeDates[date]=l[self.produit]+"/"+date
        return ListeDates
    def listeDates(self):
        liste={}
        lToutesDates=self.listeToutesDates()
        for date in self.calendrier():
            try:
                liste[date]=lToutesDates[date]
            except KeyError:
                pass                #N'ajoute pas la clée dans le dict retourné
        return liste
    def calendrier(self):
        date=datetime.date(int(self.date[:4]),int(self.date[5:7]),int(self.date[8:10]))
        for i in range(self.delta):
            yield "{:%Y.%m.%d}".format(date+datetime.timedelta(days=i))
    def authentification(self,session):
        #Devrait éventuellement déterminer lorsque l'authentification ne réussi pas
        def authentificationUSGS():
            h=session.get("https://e4ftl01.cr.usgs.gov/MOLT/MOD09A1.005/2000.02.18/MOD09A1.A2000049.h00v08.005.2006268222532.hdf",auth=(self.utilisateur,self.motdepasse),allow_redirects=False)
            h=session.get(h.headers["Location"],auth=(self.utilisateur,self.motdepasse),allow_redirects=False)
        def authentificationNSIDC():
            form={}
            formfield=["utf8","authenticity_token","client_id","redirect_uri","commit"]#,"response_type","state","stay_in"
            r=session.get("https://urs.earthdata.nasa.gov")#
            rs=r.text
            
            for name in formfield:
                utf8formA=rs.find('name="'+name+'"')
                utf8formA+= rs[utf8formA:].find('value=\"') +len('value=\"')
                utf8formB=utf8formA+rs[utf8formA:].find('\"')

                form[name]=rs[utf8formA:utf8formB]
        
#print form
            form["username"]=self.utilisateur
            form["password"]=self.motdepasse
            h=session.post('https://urs.earthdata.nasa.gov/login',data=form)
            h=session.get(URL+"/MOST")
            import time
            time.sleep(3)
            #print(h.text)
            #a=h.text.find('redirectURL = "')+len('redirectURL = "')
            
            #b=h.text[a:].find('"')+a
            #try:
                #session.get(h.text[a:b])
            #except requests.exceptions.ConnectionError:
                #time.sleep(1)
                #session.get(h.text[a:b])
        authentificationNSIDC()
        authentificationUSGS()
    def listefichiersATelecharger(self,addresseDate):
        listefichiers={}
        typeOfFile=["\.hdf","\.hdf\.xml"]
        #Get list of file for date
        r=self.session.get(addresseDate)
        rs=r.text
        for tuile in self.tuiles:
            for typ in typeOfFile:
                try:
                    #Regex to find the file to download
                    prog=re.compile('href="[A-Z,a-z,0-9,_,\.]{1,}'+tuile+'[A-Z,a-z,0-9,_,\.]{1,}'+typ)
                    #Find first occurence of this re
                    m=prog.search(rs)
                    listefichiers[m.group(0)[6:]]=addresseDate+"/"+m.group(0)[6:]
                except:
                    pass
        return listefichiers
    def telechargerUnfichier(self,addresse,nom):
        #Devrait vérifier l'existance d'un fichier sur le disque avant de le télécharger
        r=self.session.get(addresse)
        with open(os.path.join(self.output, nom),'wb') as f:
            tailleFichier = int(r.headers['content-length'])#Ça va récupérer la taille du fichier à télécharger
            f.write(r.content)
        reussi= tailleFichier==os.path.getsize(os.path.join(self.output, nom))
        #Si le fichier télécharger a la taille théorique alors celle-ci est retournée.
        if reussi:
            return tailleFichier
        else:
            return None

    def telechargerTout(self):
        echecDeSuite=0
        
        while True:
            try:
                #On va chercher toutes les dates disponible sur le serveur de la NSIDC pour un produit en particulier
                self.log.log('c',Nom,u'Lister les Dates disponibles sur le serveur.')
                ListeDate=self.listeDates()
                self.log.log('r',Nom,u'Lister les Dates disponibles sur le serveur.')
                #Chaque date est entrée comme clé dans un dictionnaire et une adresse du dossier de cette date y est assignée
                self.log.log('i',Nom,str(len(ListeDate))+u' date(s) ont été trouvé.')
                LListeDate=list(ListeDate)
                break
            except:
                echecDeSuite+=1
                if echecDeSuite>3:
                    break
            
        echecDeSuite=0
        for da in self.ignore:
            try:
                LListeDate.remove(da)
            except ValueError:
                pass
        for date in LListeDate:
            listeTrucsTelecharges=[]
            image=imageObject.imageModis([])
            #Pour chacune des dates, Une liste des adresse de téléchargement est produite
            try:
                self.log.log('c',Nom,u'Lister les fichier de téléchargement disponibles pour '+date)
                #Créer la liste des fichier à télécharger
                dic=self.listefichiersATelecharger(ListeDate[date])
                listFile=[imageObject.Afile(name,date,dic[name],name.split(".")[-1],os.path.join(self.output, name)) for name in dic]
                #Créer l'image qui sera retournée
                image=imageObject.imageModis(listFile)
                ListeFichier=image.files
                self.log.log('r',Nom,u'Lister les fichier de téléchargement disponibles pour '+date)
                self.log.log('i',Nom,str(len(ListeFichier))+u' fichier(s) ont été trouvés pour '+date)
                for fichier in ListeFichier:
                    self.log.log('c',Nom,u'Télécharment pour le fichier '+fichier.name+u' du '+date)
                    
                    r=self.telechargerUnfichier(fichier.link,fichier.name)
                    if not r==None:
                        listeTrucsTelecharges.append(fichier.name)
                        self.log.log('r',Nom,u'Télécharment pour le fichier '+fichier.name+u' du '+date)
                        self.log.log('i',Nom,u'Le fichier '+fichier.name+u' du '+date+u' a une taille de '+str(r))
                    else:
                        self.log.log('e',Nom,u'Télécharment pour le fichier '+fichier.name+u' du '+date)
                    if self.listfile==True:
                        # Write a file list file
                        with open(os.path.join(self.output, 'listfile'+self.produit.upper()+'.txt'),'a') as f:
                            f.write(fichier.name+'\n')
                echecDeSuite=0
                yield image
            except requests.exceptions.ConnectionError:
                echecDeSuite+=1
                if echecDeSuite<3:
                    LListeDate.append(date)
                
                yield image#date, listeTrucsTelecharges
def main():
    for x in telecharger("mod10a2.006","user","password",date="2010-02-20",delta=20,tuiles=['h12v04','h13v04'],output="test/").telechargerTout():
        print(x.files)
if __name__=='__main__':
    main()
    



from .Downloader import telecharger as downloader
from . import logMod
from . import imageObject
from . import Downloader
from .mosaic import mosaic
from .clip import clip
from .subset import subset
from .Compress import compress
from .imageObject import imageModis
from .imageObject import Afile
from . import convert



# coding: utf8
# Importations
from pyhdf.SD import *
import pyhdf.error 
from pyhdf.V import *
from pyhdf.HDF import *
from pyhdf.VS import *
import re
import numpy as np
from modisSuite import logMod

Nom="modisSuite Mosaic"
# Defining the class to manage attributes from the HDF file
class atribute:
	def __init__(self,raw,name="",dsi=[0,]):
		#print(dsi)
		self.name=name
		self.raw=raw
		self.GROUP = {}
		self.OBJECT = {}
		self.OBJECTindex = {}
		self.variable = {}
		colect=""
		colecting=False
		end=""
		lines=self.raw.split('\n')
		progb=re.compile('GROUP=[A-Za-z_0-9]{1,}')
		proge=re.compile('END_GROUP=[A-Za-z_0-9]{1,}')
		progob=re.compile('OBJECT=[A-Za-z_0-9]{1,}')
		progoe=re.compile('END_OBJECT=[A-Za-z_0-9]{1,}')
		for line in lines:
			#Add group
			b=progb.findall(line)
			e=proge.findall(line)
			ob=progob.findall(line)
			oe=progoe.findall(line)
			if (len(e)==0 and len(b)>0) and (not colecting):
				# Begin collecting
				colecting=True
				colect=""
				end=b[0]
				#print(type(e),type(b),type(line))
			elif (len(oe)==0 and len(ob)>0) and (not colecting):
				# Begin collecting
				colecting=True
				colect=""
				end=ob[0]
				#print(type(e),type(b),type(line))
			elif len(e)>0 and len(b)>0 and line.find(end)>-1:
				colecting=False
				# End collecting
				name=end.split("=")[-1]
				self.GROUP[name]=atribute(colect,dsi=dsi)
			elif len(oe)>0 and len(ob)>0 and line.find(end)>-1:
				colecting=False
				# End collecting
				name=end.split("=")[-1]
				self.OBJECTindex[name] = dsi[0]
				dsi[0]+=1
				self.OBJECT[name] = atribute(colect,dsi=dsi)
			elif colecting:
				# Collecting
				# print(line)
				colect+=line+"\n"
			elif  len(b)==0 and len(ob)==0:
				# adding to variable
				try:
					#print(line)
					self.variable[line.split("=")[0].strip()]=line.split("=")[1]
				except IndexError:
					pass
			
	def listgrid(self):
	# list the grids in the group
		try:
			gs=self.GROUP["GridStructure"]
			gg=list(gs.GROUP.keys())
			return len(gg),gg
		except KeyError:
			return -1,[]
	def listgridname(self):
		# list the grids name in the group
		try:
			gs=self.GROUP["GridStructure"]
			gg=[gs.GROUP[x].variable["GridName"] for x in gs.GROUP]
			return len(gg),gg
		except KeyError:
			return -1,[]
	def listdataset(self):
		#Aller chercher les grids
		gr=self.listgrid()[1]
		ds=[]
		for g in gr:
			ds+=list(self.GROUP["GridStructure"].GROUP[g].GROUP["DataField"].OBJECT.keys())
		return ds
	def listdatasetbyname(self):
		#get ds by name
		gr=self.listgrid()[1]
		ds=[]
		for g in gr:
			dsg=self.GROUP["GridStructure"].GROUP[g].GROUP["DataField"].OBJECT
			ds+=[dsg[x].variable["DataFieldName"] for x in dsg]
		return ds
	def getgridbyname(self,name):
		gs=self.GROUP["GridStructure"]
		for x in gs.GROUP:
			if gs.GROUP[x].variable["GridName"] == name:
				return gs.GROUP[x]
		return None
	def orderedDS(self):
		#get ds by name
		gr=self.listgrid()[1]
		ds=self.listdatasetbyname()
		#print(gr)
		for g in gr:
			dsg = self.GROUP["GridStructure"].GROUP[g].GROUP["DataField"]
			i = dsg.OBJECTindex
			ii = dsg.OBJECT
			#print("haha",ds)
			for e in i:
				#print(i[e])
				ds[i[e]] = ii[e].variable["DataFieldName"]
		return ds
	def __str__(self):
		st=""
		# Ajouter les variables
		for x in self.variable:
			st+=x+"="+self.variable[x]+"\n"
		# Ajouter les group
		
		for x in self.GROUP:
			st+="GROUP="+x+"\n"
			
			gg=str(self.GROUP[x]).split("\n")
			gg.remove("")
			st+="\n".join(["\t"+h for h in gg])+"\n"
			st+="END_GROUP="+x+"\n"
		# Ajouter les object
		for x in self.OBJECT:
			st+="OBJECT="+x+"\n"
			gg=str(self.OBJECT[x]).split("\n")
			gg.remove("")
			st+="\n".join(["\t"+h for h in gg])+"\n"
			st+="END_OBJECT="+x+"\n"
		return st
        
        

def mosaic(*arg,**args):
	# This function will take files tranfered in *arg and will mosaic them together and produce a new file
	# mosaic(file1,[file2,file3,...],endfile)
	# If only file1 and endfile is given, the function will only produce a copy without any other modification
	try:
		log=args["log"]
	except KeyError:
		log=logMod.Log("",nolog=True)
	if len(arg)>2:
		lfile = arg[:-1]				# This is the list of the NAME of the files to merge
		newfilename = arg[-1]			# This is the final file NAME
		# Should eventually check if files exists and can be read ***IMPROVE***
		lfHDF = []						# This is the list of the FILES to merge
		latt = []							# This is the list of the ATTRIBUTE "StructMetadata.0" of the files
		for fil in lfile:
			try:
				a=SD(fil,SDC.READ)
			except TypeError:
				a=SD(fil.encode('ascii','ignore'),SDC.READ)
			lfHDF.append(a)
			#print("hoho")
			latt.append(atribute(lfHDF[-1].attributes()["StructMetadata.0"],fil,dsi=[0,]))
			
		
		
		## Listing all the GRIDS that the new file will have
		gridlist = []						# This is the list of GRIDS to include in the final file

		for attOfF in latt:
			# Should check if any grid ***IMPROVE***
			gridlist += attOfF.listgridname()[1]		# listgridname return a list of all the grids name

		# remove double entry
		gridlist = list(set(gridlist))


		## Listing all the DATASETS that the new file will have
		dslist = []						# This is the list of DATASETS to include in the final file
		for attOfF in latt:
			# Should check if any grid ***IMPROVE***
			dslist = attOfF.orderedDS()
			
		# remove double entry
		# dslist = list(set(dslist))


		
		## Validation of commoun information
###############################################################################
# Some informations have to be the same for each file or else no mosaic can   #
# be made for exemple two files with not the same projection type can't be    #
# merged together. ***IMPROVE*** Maybe in the future we could transform file  #
# so that they have the same projection or some other thing.                  #
###############################################################################

		# List of parameter to check to insure that they are the same
		paramMustSim = ["Projection","ProjParams","SphereCode"]
		# Dictionary that will keep all the informations about every file
		paramMustSimDict = {}

		for grid in gridlist:
			# Verification of a grid

			first = True			# Variable that will enable the construction of the dict
			paramdict = {}		# Dictionary that keep the actual value that have to be the same
			
			for attOfF in latt:
				# Verification of a file
				bigG = attOfF.getgridbyname(grid)		# Getting all the attributes in the grid of a file
				if bigG is not None:
					# If the grid exists in that file
					if first :
						# If this is the first time that a file is check for that grid
						first = False
						for p in paramMustSim:
							# Checking every parameters that must be the same
							paramdict[p] = bigG.variable[p]
							
						# Validation of same Dtype for each datafield
						go = bigG.GROUP["DataField"].OBJECT
						for r in go:
							paramdict[go[r].variable["DataFieldName"]]=go[r].variable["DataType"]
					else:
						# If it's not the first time that a file is check for that grid
						for p in paramMustSim:
							# Checking every parameters that must be the same
							if not paramdict[p]==bigG.variable[p]:
								# Stop merging and return error ***IMPROVE*** 
								# Maybe do only the things that can be done ***IMPROVE***
								log.log('e',Nom,"Error dataset are not compatible")
								
						# Validation of same Dtype for each datafield
						go=bigG.GROUP["DataField"].OBJECT
						for r in go:
							if not paramdict[go[r].variable["DataFieldName"]]==go[r].variable["DataType"]:
								# Stop merging and return error ***IMPROVE*** 
								# Maybe do only the things that can be done ***IMPROVE***
								log.log('e',Nom,"Error dataset are not compatible")
								
			# Keep all this info for later it's going to be useful
			paramMustSimDict[grid]=paramdict
				
				



		## Determination of new informations
###############################################################################
# Some properties have to be calculated in order to merge. This section is    #
# doing just that                                                             #
###############################################################################

		gridResolX={}			# Getting the RESOLUTION in the X direction for each grid
		gridResolY={}			# Getting the RESOLUTION in the Y direction for each grid
		extremeup={}			# Getting the UPPER coordinates for each grid
		extremedown={}			# Getting the LOWEST coordinates for each grid
		extremeleft={}			# Getting the LEFTMOST coordinates for each grid
		extremeright={}			# Getting the RIGHTMOST coordinates for each grid
		gridDimX={}				# Getting the DIMENSIONS of X direction for each grid
		gridDimY={}				# Getting the DIMENSIONS of Y direction for each grid
		NoValueDS={}			# Getting the fill value of each dataset
		dtypeDS={}				# Getting the DTYPE for each dataset
		dstogrid={}				# Knowing wich is the GRID for each dataset
		filGridULC={}			# Getting the upper left corner of each file for each grid

		for grid in gridlist:
			# For each grid
			filGridULC[grid]={}			# Adding a dictionary for each grid that will contain information on every file
			for attOfF in latt:
				### Determination of resolution of each grid
				# ***IMPROVE*** Should check if bigd is none
				bigG=attOfF.getgridbyname(grid)				# Getting all the attributes in the grid of a file
				
				# Get extreme grid point
				ulp=eval(bigG.variable["UpperLeftPointMtrs"])
				lrp=eval(bigG.variable["LowerRightMtrs"])
				
				# Get grid dimmension
				dimx=int(bigG.variable["XDim"])
				dimy=int(bigG.variable["YDim"])
				
				# Calculate grid resolution
				gridResolX[grid]=(lrp[0]-ulp[0])/dimx
				gridResolY[grid]=(ulp[1]-lrp[1])/dimy
				
				### Determination of new extreme coordinates for each grid
				# up
				try:
					if extremeup[grid]< ulp[1]:
						extremeup[grid]=ulp[1]
				except KeyError:
					extremeup[grid]=ulp[1]
				# down
				try:
					if extremedown[grid]> lrp[1]:
						extremedown[grid]=lrp[1]
				except KeyError:
					extremedown[grid]=lrp[1]
				# left
				try:
					if extremeleft[grid]> ulp[0]:
						extremeleft[grid]=ulp[0]
				except KeyError:
					extremeleft[grid]=ulp[0]
				# right
				try:
					if extremeright[grid]< lrp[0]:
						extremeright[grid]=lrp[0]
				except KeyError:
					extremeright[grid]=lrp[0]
				### Detetermination of dataset to grid name
				if bigG is not None:
					go=bigG.GROUP["DataField"].OBJECT
					for r in go:
						dstogrid[ go[r].variable["DataFieldName"] ] = grid
				## Determination of ULC for each grid in each file
				filGridULC[grid][attOfF.name] = ulp
			## determination of new dimension for each grid
			gridDimY[grid] = int((extremeup[grid]-extremedown[grid])/gridResolY[grid])
			gridDimX[grid] = int((extremeright[grid]-extremeleft[grid])/gridResolX[grid])

		for ds in dslist:
			# For each dataset
			for sd in lfHDF:
				# For each hdf file
				
				# Try opening dataset
				try:
					sds = sd.select(eval(ds))
					# Get fill value
					NoValueDS[ds] = sds.getfillvalue()
					# Get dtype
					dtypeDS[ds] = sds.info()[3]
				except:
					log.log('e',Nom,"no dataset")



		## Start creating new file
###############################################################################
# This is the actual part were stuf appens                                    #
###############################################################################

		# This part is the same for every file in any circumstances
		########## absolute ########################
		
		# Open new file
		try:
			hdf = HDF(newfilename, HC.WRITE  | HC.CREATE  |HC.TRUNC)
			sd  =  SD(newfilename, SDC.WRITE | SDC.CREATE )
		except TypeError:
			hdf = HDF(newfilename.encode('ascii','ignore'), HC.WRITE  | HC.CREATE  |HC.TRUNC)
			sd  =  SD(newfilename.encode('ascii','ignore'), SDC.WRITE | SDC.CREATE )
		
		v=hdf.vgstart()
		vg={}
		vg1={}
		vg2={}
		
		## rewrite the gridlist
		gridlist = []
		for ds in dslist:
			if dstogrid[ds] not in gridlist:
				gridlist.append(dstogrid[ds])
				
		for grid in gridlist:
			vg[grid]=v.attach(-1,write=1)
			vg[grid]._class="GRID"
			vg[grid]._name=eval(grid)
			vg1[grid]=v.attach(-1,write=1)
			vg2[grid]=v.attach(-1,write=1)
			vg1[grid]._class="GRID Vgroup"
			vg1[grid]._name="Data Fields"
			vg2[grid]._class="GRID Vgroup"
			vg2[grid]._name="Grid Attributes"
			vg[grid].insert(vg1[grid])
			vg[grid].insert(vg2[grid])
		########## absolute ########################


		# Create dataset with the right size
		for ds in dslist:
			theGrid=dstogrid[ds]
			# Get grid name of data set
			sds = sd.create(eval(ds),dtypeDS[ds],(gridDimY[theGrid],gridDimX[theGrid]))
			
			# Set fill value
			fv=NoValueDS[ds]
			try:
				sds.setfillvalue(NoValueDS[ds])
			except OverflowError:
				log.log('e',Nom,"setfillvalue")
				sds.setfillvalue(0)
			## write real data
			for fil in range(len(latt)):
				try:
					# Determine were the data will be writen
					ulc = filGridULC[theGrid][latt[fil].name]
					# Determine the position on the grid
					y = (extremeup[theGrid]-ulc[1])/(extremeup[theGrid]-extremedown[theGrid])
					x = (ulc[0]-extremeleft[theGrid])/(extremeright[theGrid]-extremeleft[theGrid])
					y = int(y*gridDimY[theGrid])
					x = int(x*gridDimX[theGrid])
					# read data from files
					osds = lfHDF[fil].select(eval(ds))
					sh = osds[:].shape
					sds[y:y+sh[0],x:x+sh[1]] = osds[:]
					osds.endaccess()
				except:
					pass
			# Close sds
			vg1[dstogrid[ds]].add(HC.DFTAG_NDG,sds.ref())
			sds.endaccess()


    
		for g in vg1:
			vg1[g].detach()
			vg2[g].detach()
			vg[g].detach()

		# Create attribute table for the file
		attstr="GROUP=GridStructure\n"
		gridcount=1
		for gr in gridlist:
			# Start group grid
			attstr+="\tGROUP=GRID_%i\n"%gridcount
			# Add grid name
			attstr+="\t\tGridName=%s\n"%gr
			# Add dimention
			attstr+="\t\tXDim=%i\n"%gridDimX[gr]
			attstr+="\t\tYDim=%i\n"%gridDimY[gr]
			# Add UpperLeftPointMtrs
			attstr+="\t\tUpperLeftPointMtrs=(%f,%f)\n"%(extremeleft[gr],extremeup[gr])
			# Add lrp
			attstr+="\t\tLowerRightMtrs=(%f,%f)\n"%(extremeright[gr],extremedown[gr])
			# Add projection
			attstr+="\t\tProjection=%s\n"%paramMustSimDict[gr]["Projection"]
			# ProjParams
			attstr+="\t\tProjParams=%s\n"%paramMustSimDict[gr]["ProjParams"]
			# SphereCode
			attstr+="\t\tSphereCode=%s\n"%paramMustSimDict[gr]["SphereCode"]

			
			attstr+="""\t\tGROUP=Dimension
		\t\tEND_GROUP=Dimension
		\t\tGROUP=DataField\n"""

			## Add data sets
			# create list of ds for current grid
			lsdsgr=[]
			dsnum=1
			for ds in dslist:
				if dstogrid[ds] == gr:
					# Add object
					attstr+="\t\t\tOBJECT=DataField_%i\n"%dsnum
					# datafield name
					attstr+="\t\t\t\tDataFieldName=%s\n"%ds
					# datatype
					attstr+="\t\t\t\tDataType=%s\n"%paramMustSimDict[gr][ds]
					# dim
					attstr+='\t\t\t\tDimList=("YDim","XDim")\n'
					attstr+="\t\t\tEND_OBJECT=DataField_%i\n"%dsnum
					dsnum+=1
			attstr+="\t\tEND_GROUP=DataField\n"
			attstr+="""\t\tGROUP=MergedFields
		\t\tEND_GROUP=MergedFields\n"""
			attstr+="\tEND_GROUP=GRID_%i\n"%gridcount
			gridcount+=1
		attstr+="""END_GROUP=GridStructure
		GROUP=PointStructure
		END_GROUP=PointStructure
		END"""
		# adding attribute to new file
		att=sd.attr('StructMetadata.0')
		att.set(SDC.CHAR,attstr)
		sd.end()
		hdf.close()
		
		# This should return something somehow
	elif len(arg)>1:
		afile = arg[0]				# This is the list of the NAME of the files to merge
		newfilename = arg[1]			# This is the final file NAME
		# Create a copy
		from shutil import copyfile
		copyfile(afile,newfilename)



 # coding: utf8
 # Importations
from pyhdf.SD import *
import pyhdf.error 
from pyhdf.V import *
from pyhdf.HDF import *
from pyhdf.VS import *
import re
import numpy as np
import os
from modisSuite import logMod

Nom="modisSuite Compress"
def compress(*arg,**karg):
	# This function compress the file with a given compress parameter
	# compress(file,comression parameter)
	
	fileName=arg[0]
	
	try:
		log=karg["log"]
	except KeyError:
		log=logMod.Log("",nolog=True)
	
	
	# Open HDF file
	try:
		sd_id = SD(fileName, SDC.WRITE )
	except TypeError:
		sd_id = SD(fileName.encode('ascii','ignore'), SDC.WRITE )
	
	
	#open evry data set
	for dsname in list(sd_id.datasets().keys()):
		sds_id = sd_id.select(dsname)
		data=np.copy(sds_id[:])
		sds_id[:]=0
		try:
			sds_id.setcompress(*arg[1])         # args depend on compression type
		except HDF4Error as msg:
			log.log('e',Nom,"Error compressing the dataset")
			sds_id.endaccess()
			sd_id.end()
			return
		sds_id[:]=data
		sds_id.endaccess()

	# Close hdf file to flush compressed data.
	sd_id.end()
	return None
